# Pentaho endpoint constants


#############################################
# User Endpoint
#############################################

PENTAHO_USERS_ENDPOINT_API = "users"

USERS_USERNAME_VAR = "userName"
USERS_PASSWORD_VAR = "password"

USERS_NEW_PASSWORD_VAR = "newPassword"
USERS_OLD_PASSWORD_VAR = "oldPassword"
USERS_DELETE_VAR = "userNames"

ROLE_CREATE_VAR = "roleName"
ROLE_DELETE_VAR = "roleNames"
ROLE_LIST_MEMBERS_VAR = "roleName"
ROLE_MAP_LOCALE_VAR = "locale"

LIST_USERS = "list_users"
CREATE_USERS = "create_users"
DELETE_USERS = "delete_users"
CHANGE_USER_PASSWORD = "change_user_password"

TAB_SEPARATOR = '\t'

# see documentation here https://help.pentaho.com/Documentation/6.0/0R0/070/010/0D0/0O0
PENTAHO_USER_ENDPOINT_DEFINITION = {
    CREATE_USERS: ("put", "/pentaho/api/userroledao/createUser"),
    LIST_USERS: ("get", "/pentaho/api/userroledao/users"),
    DELETE_USERS: ("put", "/pentaho/api/userroledao/deleteUsers"),
    CHANGE_USER_PASSWORD: ("put", "/pentaho/api/userroledao/user")
}

#############################################
# Role Endpoint
#############################################

PENTAHO_ROLES_ENDPOINT_API = "roles"

ROLES_ASSIGN_VAR = "roleNames"

GET_ROLES_FOR_USER = "list_user_roles"
ASSIGN_ROLE_TO_USER = "assign_role_to_user"
REMOVE_ROLE_FROM_USER = "remove_role_from_user"
CREATE_ROLE = "create_role"
DELETE_ROLE = "delete_role"
LIST_ROLES = "list_roles"
LIST_MEMBERS_ROLE = "list_members_role"
ASSIGN_PERM_TO_ROLE = "assign_perm_to_role"
LIST_PERM_ROLE_MAP = "perm_role_map"

PENTAHO_ROLE_ENDPOINT_DEFINITION = {
    GET_ROLES_FOR_USER: ("get", "/pentaho/api/userroledao/userRoles"),
    ASSIGN_ROLE_TO_USER: ("put", "/pentaho/api/userroledao/assignRoleToUser"),
    REMOVE_ROLE_FROM_USER: ("put", "/pentaho/api/userroledao/removeRoleFromUser"),
    CREATE_ROLE: ("put", "/pentaho/api/userroledao/createRole"),
    DELETE_ROLE: ("put", "/pentaho/api/userroledao/deleteRoles"),
    LIST_ROLES: ("get", "/pentaho/api/userroledao/roles"),
    LIST_MEMBERS_ROLE: ("get", "/pentaho/api/userroledao/roleMembers"),
    ASSIGN_PERM_TO_ROLE: ("put", "/pentaho/api/userroledao/roleAssignments"),
    LIST_PERM_ROLE_MAP: ("get", "/pentaho/api/userroledao/logicalRoleMap")
}

#############################################
# Files Endpoint
#############################################

PENTAHO_FILES_ENDPOINT_API = "files"

CREATE_DIRECTORY = "create_directory"
LIST_DIRECTORY = "list_directory"
GET_ACL = "get_acl"
SET_ACL = "set_acl"

PENTAHO_FILE_ENDPOINT_DEFINITION = {
    CREATE_DIRECTORY: ("put", "/pentaho/api/repo/dirs/"),
    LIST_DIRECTORY: ("get", "/pentaho/api/repo/files/"),
    GET_ACL: ("get", "/pentaho/api/repo/files/"),
    SET_ACL: ("put", "/pentaho/api/repo/files/")
}

#############################################
# Pentaho class constant
#############################################

PENTAHO_BASIC_AUTH = 'basic'
PENTAHO_COOKIE_AUTH = 'cookie'
PENTAHO_DEFAULT_COOKIE_NAME = 'JSESSIONID'

PENTAHO_AVAILABLE_ENDPOINT_TYPE = {
    PENTAHO_USERS_ENDPOINT_API: PENTAHO_USER_ENDPOINT_DEFINITION,
    PENTAHO_ROLES_ENDPOINT_API: PENTAHO_ROLE_ENDPOINT_DEFINITION,
    PENTAHO_FILES_ENDPOINT_API: PENTAHO_FILE_ENDPOINT_DEFINITION
}



# -*- coding: utf-8 -*-
from __future__ import absolute_import

from penapi.pentaho import (
    Pentaho,
    PentahoBasicAuth,
    PentahoCookieAuth,
    PENTAHO_DEFAULT_COOKIE_NAME
)


_pentaho_obj = Pentaho()

users = _pentaho_obj.users
roles = _pentaho_obj.roles
files = _pentaho_obj.files



def set_basic_auth_method(username, password):
    _pentaho_obj.set_auth_method(PentahoBasicAuth(username, password))


def set_cookie_auth_method(cookie_value, cookie_name=PENTAHO_DEFAULT_COOKIE_NAME):
    _pentaho_obj.set_auth_method(PentahoCookieAuth(cookie_value, cookie_name))


def renew_cookie_value(cookie_value, cookie_name=PENTAHO_DEFAULT_COOKIE_NAME):
    set_cookie_auth_method(cookie_value, cookie_name)


def set_pentaho_base_url(pentaho_base_url):
    _pentaho_obj.set_pentaho_base_url(pentaho_base_url)


def set_ssl_check(ssl_check):
    _pentaho_obj.set_ssl_check(ssl_check)



import logging
import xmltodict
import json

from constants import (
    PENTAHO_FILES_ENDPOINT_API,
    CREATE_DIRECTORY,
    LIST_DIRECTORY,
    GET_ACL,
    SET_ACL
)

from penapi.base_api import PentahoBaseAPI

logger = logging.getLogger(__name__)


class PentahoFilesAPI(PentahoBaseAPI):

    def directory_to_url_path(self, directory):
        return directory.replace('/', ':').replace('\\', ':')

    def create_directory(self, path):
        """
        Create a directory
        :param path: full directory's path
        :return: success/failure
        :rtype: bool
        """
        response = self._pentaho.make_call(
            PENTAHO_FILES_ENDPOINT_API,
            CREATE_DIRECTORY,
            url_addition=self.directory_to_url_path(path)
        )
        return response.status_code == 200 or response.status_code == 409

    def list_directory(self, path, depth=None):
        """
        Get directory file tree
        :param pentaho: the directory tree
        :return: success/failure
        :rtype: list
        """
        dir_tree = []
        url_addition = "{}/tree".format(self.directory_to_url_path(path))
        if type(depth) is int:
            url_addition = "{}?depth={}".format(url_addition, depth)
        response = self._pentaho.make_call(
            PENTAHO_FILES_ENDPOINT_API,
            LIST_DIRECTORY,
            url_addition=url_addition)
        if response.status_code == 200:
            try:
                dir_tree = xmltodict.parse(response.text)['repositoryFileTreeDto']
            except Exception, e:
                logger.exception(e)
        return dir_tree

    def get_acl(self, path):
        """
        Get ACL for a specific path/file
        :param path: path to get ACL
        :return: dict with ACL parameters
        """
        url_addition = "{}/acl".format(self.directory_to_url_path(path))
        response = self._pentaho.make_call(
            PENTAHO_FILES_ENDPOINT_API,
            GET_ACL,
            url_addition=url_addition)
        acl = False
        if response.status_code == 200:
            try:
                acl = xmltodict.parse(response.text)['repositoryFileAclDto']
            except Exception, e:
                logger.exception(e)
        return acl

    def set_acl(self, path, permissions):
        """
        Set ACL for a specific path/file
        :param path: path to set ACL
        :param permissions: dict with ACL parameters format
                Example of permission dictionnary:
                {
                    "entriesInheriting": False,
                    "owner": "test@test.com",
                    "ownerType": "1",
                    "aces": [
                        {
                            "modifiable": True,
                            "recipientType": 1,
                            "permissions": [ 0 ],
                            "recipient": "Authenticated"
                        },
                        {
                            "modifiable": True,
                            "recipientType": 1,
                            "permissions": [ 4 ],
                            "recipient": "Administrator"
                        }
                    ]
                }
        :return: boolean true/false if operation succeeded
        """
        url_addition = "{}/acl".format(self.directory_to_url_path(path))
        response = self._pentaho.make_call(
            PENTAHO_FILES_ENDPOINT_API,
            SET_ACL,
            url_addition=url_addition,
            json=permissions
        )
        return response.status_code == 200



from distutils.util import strtobool
from constants import (
    PENTAHO_DEFAULT_COOKIE_NAME,
    PENTAHO_AVAILABLE_ENDPOINT_TYPE
)
from users_api import PentahoUsersAPI
from roles_api import PentahoRolesAPI
from files_api import PentahoFilesAPI
import requests
import urlparse


class PentahoAuth(object):

    def get_auth_kwarg(self):
        raise NotImplementedError()


class PentahoBasicAuth(PentahoAuth):

    def __init__(self, username, password):
        self._username = username
        self._password = password

    def get_auth_kwarg(self):
        return {'auth': (self._username, self._password)}


class PentahoCookieAuth(PentahoAuth):

    def __init__(self, cookie_value, cookie_name=PENTAHO_DEFAULT_COOKIE_NAME):
        self._cookie_name = cookie_name
        self._cookie_value = cookie_value

    def get_auth_kwarg(self):
        return {'cookies': {self._cookie_name: self._cookie_value}}


class Pentaho(object):

    def __init__(self, pentaho_base_url="http://localhost", pentaho_auth_method=None, ssl_check=True):
        self.pentaho_base_url = pentaho_base_url
        self.do_ssl_check = ssl_check if type(ssl_check) is bool else strtobool(ssl_check)
        self.pentaho_auth_method = pentaho_auth_method
        self.users = PentahoUsersAPI(self)
        self.roles = PentahoRolesAPI(self)
        self.files = PentahoFilesAPI(self)

    def set_auth_method(self, pentaho_auth_method):
        self.pentaho_auth_method = pentaho_auth_method

    def set_pentaho_base_url(self, pentaho_base_url):
        self.pentaho_base_url = pentaho_base_url

    def set_ssl_check(self, ssl_check):
        self.do_ssl_check = ssl_check

    @staticmethod
    def validate_urls(url):
        """
        Validate a url
        :param url: the url to validate
        :return: True if valid, False otherwise
        """
        valid = False
        parsed_url = urlparse.urlparse(url)
        if parsed_url:
            valid = True
        return valid

    def get_auth_kwarg(self):
        """
        return authentication parameters for requests object
        :return: dict containing authentication parameters
        """
        if self.pentaho_auth_method:
            return self.pentaho_auth_method.get_auth_kwarg()
        return {}

    def make_call(self, endpoint_type, endpoint, url_addition='', **kwargs):
        """
        Call Pentaho REST endpoint
        :param endpoint_type: type of endpoint to call (users, roles, etc...)
        :param endpoint: which endpoint to call (list, create, etc...)
        :param kwargs: parameters for the requests
        :return Response object from pentaho
        """
        kwargs.update(**self.get_auth_kwarg())
        if endpoint_type not in PENTAHO_AVAILABLE_ENDPOINT_TYPE:
            raise ValueError("[ERROR] api endpoint type {} does not exist".format(endpoint_type))
        method, url = self.get_endpoint_method_url(PENTAHO_AVAILABLE_ENDPOINT_TYPE[endpoint_type], endpoint)
        return requests.request(method, url + url_addition,
                                verify=self.do_ssl_check, **kwargs)

    def get_endpoint_method_url(self, type, endpoint):
        """
        Generate based url
        :param endpoint: the type of endpoint required
        :param type: the type api
        :return: (method, url)
        """
        if not endpoint or endpoint not in type:
            raise ValueError("[ERROR] get_endpoint_method_url::endpoint_type improper definition... ")
        endpoint_method, endpoint_url = type[endpoint]
        pentaho_endpoint = urlparse.urljoin(self.pentaho_base_url, endpoint_url)
        if not self.validate_urls(pentaho_endpoint):
            raise ValueError("[ERROR] Improper endpoint :: {} ... ".format(pentaho_endpoint))
        return endpoint_method, pentaho_endpoint
